//------------------------------------------------------------------------------
// GPL LICENSE (short)
//------------------------------------------------------------------------------
/*
 * Copyright (c) 2020 R1KO, vadrozh, CrazyHackGUT aka Kruzya

 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 * ChangeLog:
		1.0.0 -	Релиз
		1.1.0 - Новый синтаксис, поддержка [VIP]Core 3.0
		1.1.1 - Поддержка нескольких копий сервера в одной папке, правки под SB/MA
		1.1.2 - Удаление прав при выгрузке плагина
*/

#pragma semicolon 1

#include <sourcemod>
#include <vip_core>

#pragma newdecls required

public Plugin myinfo =
{
	name = "[VIP] Admin",
	author = "R1KO, vadrozh, -",
	description = "Выдача группы VIP по админ-группе/флагам SM",
	version = "1.1.1",
	url = "https://hlmod.ru"
};

Handle 	g_hUsers;
bool 	g_bPluginVIP[MAXPLAYERS+1];
char	g_szConfigPath[PLATFORM_MAX_PATH];

public void OnPluginStart() { BuildPath(Path_SM, g_szConfigPath, sizeof(g_szConfigPath), "data/vip/cfg/vip_admin_users.ini"); }
public void OnPluginEnd() { removeVIPGroups(); }

public void OnMapStart()
{
	if(g_hUsers)
		CloseHandle(g_hUsers);

	g_hUsers = CreateKeyValues("VIP_ADMIN_USERS");
	if(FileToKeyValues(g_hUsers, g_szConfigPath) == false)
		SetFailState("Не удалось открыть файл \"%s\"", g_szConfigPath);

	KvRewind(g_hUsers);
	
	char sBuffer[64];
	if (KvJumpToKey(g_hUsers, "ADMIN_FLAGS") && KvGotoFirstSubKey(g_hUsers))
	{
		int i, iLen;
		while (KvGotoNextKey(g_hUsers)) {
			KvGetSectionName(g_hUsers, sBuffer, sizeof(sBuffer));
			iLen = strlen(sBuffer);
			
			for(i = 0; i < iLen; ++i)
			sBuffer[i] = CharToLower(sBuffer[i]);

			KvSetNum(g_hUsers, "flags_bin", ReadFlagString(sBuffer));
		}
	}

	KvRewind(g_hUsers);

	// Код кеширования идентификаторов был удалён в связи
	// с проблемами при хранении групп в базе.
	//
	// (c) CrazyHackGUT
}

void removeVIPGroups()
{
	for (int iClient = 1; iClient <= MaxClients; ++iClient)
		if (IsClientInGame(iClient) && !IsFakeClient(iClient) && g_bPluginVIP[iClient] && VIP_IsClientVIP(iClient))
			VIP_RemoveClientVIP2(0, iClient, false, false);
}

public void OnClientPostAdminCheck(int iClient)
{
	g_bPluginVIP[iClient] = false;
	CreateTimer(2.0, Timer_Delay, GetClientUserId(iClient), TIMER_FLAG_NO_MAPCHANGE);
}

Action Timer_Delay(Handle hTimer, int iClient)
{
	if ((iClient = GetClientOfUserId(iClient)) && !VIP_IsClientVIP(iClient))
	{
		KvRewind(g_hUsers);
		if (KvJumpToKey(g_hUsers, "ADMIN_FLAGS", false) && ReadFlags(iClient))
			LoadVIPClient(iClient);
		
		KvRewind(g_hUsers);
		if (KvJumpToKey(g_hUsers, "ADMIN_GROUPS", false) && ReadGroups(iClient))
			LoadVIPClient(iClient);
	}
}

bool ReadFlags(int iClient)
{
	if (KvGotoFirstSubKey(g_hUsers))
	{
		int iFlags = GetUserFlagBits(iClient);
		if(iFlags)
			while (KvGotoNextKey(g_hUsers))
				if (iFlags & KvGetNum(g_hUsers, "flags_bin"))
					return true;
	}

	KvRewind(g_hUsers);
	
	return false;
}


bool ReadGroups(int iClient)
{
	AdminId iAdminID = GetUserAdmin(iClient);
	if (iAdminID != INVALID_ADMIN_ID)
	{
		int iGroups = GetAdminGroupCount(iAdminID);
		if(iGroups)
		{
			char sGroup[64];
			for (int i = 0; i < iGroups; ++i)
				if (GetAdminGroup(iAdminID, i, sGroup, sizeof(sGroup)) != INVALID_GROUP_ID && KvJumpToKey(g_hUsers, sGroup))
					return true;
		}
	}

	KvRewind(g_hUsers);
	return false;
}

void LoadVIPClient(int iClient)
{
	char sGroup[64];
	KvGetString(g_hUsers, "vip_group", sGroup, sizeof(sGroup));
	VIP_GiveClientVIP(0, iClient, 0, sGroup, false);
	g_bPluginVIP[iClient] = true;
}

public void OnRebuildAdminCache(AdminCachePart part)
{
	removeVIPGroups();
	CreateTimer(0.5, Timer_RebuildDelay, _, TIMER_FLAG_NO_MAPCHANGE);
}

public Action Timer_RebuildDelay(Handle hTimer)
{
	for (int iClient = 1; iClient <= MaxClients; ++iClient)
		if (IsClientInGame(iClient) && !IsFakeClient(iClient))
			Timer_Delay(INVALID_HANDLE, GetClientUserId(iClient));

	return Plugin_Stop;
}